#!/bin/bash
#
# Demonstration on how to use Glade and the GTK-server with BASH.
#
# Tested with GTK-server 2.1.3 on Zenwalk 4.2 and BASH 3.1
# January 16, 2007 (c) Peter van Eerten - GPL license.
#
# Changed for GTK-server 2.1.4 at july 2, 2007 - PvE.
# Fixed bug with division by zero at october 13, 2007 - PvE.
# Fixed search path for Glade file at april 17, 2008 - PvE.
# Beautification for assigning returnvalues to vars at april 26, 2008 - PvE.
#
#----------------------------------------------------------- Setup embedded GTK

# Pipe filename must be unique for your application
PIPE="/tmp/gtk.bash.\$$"

# Find GTK-server configfile first
if [[ -f gtk-server.cfg ]]; then
    CFG=gtk-server.cfg
elif [[ -f /etc/gtk-server.cfg ]]; then
    CFG=/etc/gtk-server.cfg
elif [[ -f /usr/local/etc/gtk-server.cfg ]]; then
    CFG=/usr/local/etc/gtk-server.cfg
else
    echo "No GTK-server configfile found! Please install GTK-server..."
    exit 1
fi
# Now create global functionnames from GTK API
if [[ ! -f $HOME/.gtk4bash || $CFG -nt $HOME/.gtk4bash ]]; then
    echo "#!/bin/bash" > $HOME/.gtk4bash
    echo "gtk-server fifo=$PIPE" >> $HOME/.gtk4bash
    while read LINE
    do
	if [[ $LINE = *FUNCTION_NAME* ]]; then
	    LINE=${LINE#*= }
	    printf "\nfunction ${LINE%%,*}\n" >> $HOME/.gtk4bash
	    printf "{\necho ${LINE%%,*} \$@ > $PIPE" >> $HOME/.gtk4bash
	    printf "\nread GTK < $PIPE\n}\n" >> $HOME/.gtk4bash
	fi
    done < $CFG
fi
# Declare global variables
declare GTK NULL="NULL"
unset CFG PIPE LINE

# Include the generated file to use embedded GTK functions
. ${HOME}/.gtk4bash

#---------------------------------------------------------- Global inits

# GTK-server must go after an error in the script
trap 'echo "gtk_server_exit" > $PIPE' ERR

# Define SIGUSR1 in GTK-server configfile to catch exit of GTK-server
trap 'exit' SIGUSR1

# Assignment function
function define() { $2 $3 $4 $5 $6 $7 $8 $9; eval $1="$GTK"; }

#---------------------------------------------------------- GUI realization

# Save my directory
MYDIR=${0%/*}

# Initialize libs
gtk_init
glade_init

# Get GUI definition
define XML glade_xml_new "$MYDIR/calculator.glade" NULL NULL
glade_xml_signal_autoconnect $XML

# Get main window ID and connect signal
glade_xml_get_widget $XML "window"
gtk_server_connect "$GTK delete-event window"

# Get button ID's and connect signals
glade_xml_get_widget $XML "button0"
gtk_server_connect "$GTK clicked button0"
glade_xml_get_widget $XML "button1"
gtk_server_connect "$GTK clicked button1"
glade_xml_get_widget $XML "button2"
gtk_server_connect "$GTK clicked button2"
glade_xml_get_widget $XML "button3"
gtk_server_connect "$GTK clicked button3"
glade_xml_get_widget $XML "button4"
gtk_server_connect "$GTK clicked button4"
glade_xml_get_widget $XML "button5"
gtk_server_connect "$GTK clicked button5"
glade_xml_get_widget $XML "button6"
gtk_server_connect "$GTK clicked button6"
glade_xml_get_widget $XML "button7"
gtk_server_connect "$GTK clicked button7"
glade_xml_get_widget $XML "button8"
gtk_server_connect "$GTK clicked button8"
glade_xml_get_widget $XML "button9"
gtk_server_connect "$GTK clicked button9"
# Operators
glade_xml_get_widget $XML "buttonC"
gtk_server_connect "$GTK clicked buttonC"
glade_xml_get_widget $XML "buttonCE"
gtk_server_connect "$GTK clicked buttonCE"
glade_xml_get_widget $XML "buttonAdd"
gtk_server_connect "$GTK clicked buttonAdd"
glade_xml_get_widget $XML "buttonMinus"
gtk_server_connect "$GTK clicked buttonMinus"
glade_xml_get_widget $XML "buttonMul"
gtk_server_connect "$GTK clicked buttonMul"
glade_xml_get_widget $XML "buttonDiv"
gtk_server_connect "$GTK clicked buttonDiv"
glade_xml_get_widget $XML "buttonEq"
gtk_server_connect "$GTK clicked buttonEq"
# Memory buttons
glade_xml_get_widget $XML "buttonMemadd"
gtk_server_connect "$GTK clicked buttonMemadd"
glade_xml_get_widget $XML "buttonMemread"
gtk_server_connect "$GTK clicked buttonMemread"
# Get entry ID
define ENTRY glade_xml_get_widget $XML "entry"
# Calculator starts with 0
gtk_entry_set_text $ENTRY 0

#---------------------------------------------------------- Init variables

# Initialize calculator state
declare STATE=0
# Initialize temp calculator cache
declare CACHE=0
# Initialize last calculator action
declare ACTION=0
# Initiliaze calculator total result
declare TOTAL=0
# Initialize MEM function
declare MEM=0

#---------------------------------------------------------- Functions

# Define action when +, -, *, or / is pressed
function operator()
{
if [[ $STATE -eq 0 ]]
then
    case $ACTION in
	1)
	    gtk_entry_get_text $ENTRY
	    let CACHE=$CACHE+$GTK
	    gtk_entry_set_text $ENTRY $CACHE;;
	2)
	    gtk_entry_get_text $ENTRY
	    let CACHE=$CACHE-$GTK
	    gtk_entry_set_text $ENTRY $CACHE;;
	3)
	    gtk_entry_get_text $ENTRY
	    let CACHE=$CACHE*$GTK
	    gtk_entry_set_text $ENTRY $CACHE;;
	4)
	    gtk_entry_get_text $ENTRY
	    if [[ $GTK -eq 0 ]]
	    then
		gtk_entry_set_text $ENTRY "ERROR"
	    else
		let CACHE=$CACHE/$GTK
		gtk_entry_set_text $ENTRY $CACHE
	    fi;;
    esac
fi
}

# Define action when 0 1 2 3 4 5 6 7 8 9 is pressed; requires argument
function number()
{
gtk_entry_get_text $ENTRY
if [[ $GTK -eq 0 || $STATE -gt 0 ]]
then
    define CACHE gtk_entry_get_text $ENTRY
    gtk_entry_set_text $ENTRY $1
else
    gtk_entry_set_text $ENTRY $GTK$1
fi
STATE=0
}

#---------------------------------------------------------- Mainloop

until [[ $EVENT = "window" ]]
do
    define EVENT gtk_server_callback "wait"

    case $EVENT in
	"buttonMemread")
	    STATE=0
	    define CACHE gtk_entry_get_text $ENTRY
	    gtk_entry_set_text $ENTRY $MEM;;
	"buttonMemadd")
	    define MEM gtk_entry_get_text $ENTRY;;
	"buttonC")
	    gtk_entry_set_text $ENTRY 0;;
	"buttonCE")
	    TOTAL=0
	    STATE=0
	    ACTION=0
	    CACHE=0
	    MEM=0
	    gtk_entry_set_text $ENTRY 0;;
	"buttonAdd")
	    operator
	    ACTION=1
	    ((STATE+=1));;
	"buttonMinus")
	    operator
	    ACTION=2
	    ((STATE+=1));;
	"buttonMul")
	    operator
	    ACTION=3
	    ((STATE+=1));;
	"buttonDiv")
	    operator
	    ACTION=4
	    ((STATE+=1));;
	"buttonEq")
	    operator
	    ACTION=0
	    ((STATE+=1));;
	"button0")
	    number 0;;
	"button1")
	    number 1;;
	"button2")
	    number 2;;
	"button3")
	    number 3;;
	"button4")
	    number 4;;
	"button5")
	    number 5;;
	"button6")
	    number 6;;
	"button7")
	    number 7;;
	"button8")
	    number 8;;
	"button9")
	    number 9;;
    esac
done

# Exit GTK
gtk_exit 0
